import { AfterContentChecked, AfterViewChecked, Component, Input, OnInit } from '@angular/core';
import { DatasetCrudService } from './services/dataset-crud.service';
import { OpenApi } from '../publishapi/class/openapi';
import { ParameterType } from '../publishapi/class/openapi-enum';
import { OpenApiTag } from '../publishapi/class/openapi-dto';
import { environment } from 'src/environments/environment.prod';
import { NbStepperComponent } from '@nebular/theme';
import { DatasetsDialogComponent } from './datasets-dialog/datasets-dialog.component';
import { MatDialog } from '@angular/material/dialog';
import { HttpClient } from '@angular/common/http';
import { HttpMethod } from '../publishapi/class/http-enum';
import { Observable } from 'rxjs';

interface RequestInfo {
  value?: string;
  result?: string;
  type?: ParameterType;
  tag?: OpenApiTag;
  required: boolean;
}

@Component({
  selector: 'app-datasets',
  templateUrl: './datasets.component.html',
  styleUrls: ['./datasets.component.scss']
})


export class DatasetsComponent implements OnInit, AfterViewChecked, AfterContentChecked {

  @Input() openApi: OpenApi;

  oldOpenApiStringified: string;

  loading = false;
  ready = false;

  stepper: NbStepperComponent;

  tagEnum = OpenApiTag;

  values: Map<string, Map<string, Map<string, RequestInfo>>>;
  previews: Map<string, any>;
  spinners: Map<string, boolean>;
  FDP_URL = environment.fdpUrl;

  requestHistorical: string[] = [];

  Object = Object;

  //urlrepo = "dataverse.ird.fr";
  //  urlrepo = "data.inrae.fr";

  constructor(
    private dataSetService: DatasetCrudService,
    public dialog: MatDialog,
    private httpClient: HttpClient
  ) { }

  get urlRepo(): string {
    return this.openApi.servers.length > 0 ? this.openApi.servers[0].url : '';
  }

  get openApiHasChanged(): boolean {
    const openApiStringified: string = JSON.stringify(this.openApi);
    if (openApiStringified === this.oldOpenApiStringified) {
      return false;
    } else {
      this.oldOpenApiStringified = openApiStringified;
      return true;
    }
  }

  toggleHistoryText(isOpen: boolean): string {
    if (isOpen) {
      return 'Hide history';
    } else {
      return 'Show history';
    }
  }

  ngOnInit() {
    this.initComponent();
  }

  ngAfterContentChecked() {
    this.initComponent();
  }

  ngAfterViewChecked() {
    window.dispatchEvent(new Event('resize'));
  }

  initComponent() {
    if (!this.openApiHasChanged) {
      return;
    }

    this.loading = false;
    this.ready = false;

    this.values = new Map<string, Map<string, Map<string, RequestInfo>>>();
    this.previews = new Map<string, any>();
    this.spinners = new Map<string, boolean>();

    this.initValueMap();
    this.updateRequestHistorical();
    this.dataSetService.resetDataset();
  }

  initValueMap() {
    for (const path of this.openApi.paths) {

      if (!this.values.has(path.pathName)) {
        this.values.set(path.pathName, new Map<string, Map<string, RequestInfo>>());
      }

      for (const request of path.requests) {
        if (!this.values.get(path.pathName).has(path.pathName)) {
          this.values.get(path.pathName).set(request.httpmethod, new Map<string, RequestInfo>());
        }

        for (const parameter of request.parameters) {
          if (!this.values.get(path.pathName).get(request.httpmethod).has(parameter.name)) {
            this.values.get(path.pathName)
              .get(request.httpmethod)
              .set(parameter.name,
                {
                  value: parameter.schema.default,
                  type: parameter.in,
                  required: parameter.required
                });
          }
        }
      }
    }
  }

  async launchRequest(pathName: string, httpMethod: string, preview: boolean, datasetId?: number) {
    if (preview) {
      this.spinners.set(pathName, true);
    }

    const parameters = this.values.get(pathName).get(httpMethod);
    let path = pathName;

    for (const [parameterName, parameterInfo] of parameters) {
      if (parameterInfo.type === ParameterType.path) {
        let value = parameterInfo.value;
        if (parameterInfo.required && datasetId != null) {
          value = datasetId.toString();
        }
        path = path.replace(`{${parameterName}}`, value);
      }
    }

    const myHeaders: any = {};

    const url = new URL(this.urlRepo + path);

    for (const [parameterName, parameterInfo] of parameters) {
      let value = parameterInfo.value;
      if (parameterInfo.required && datasetId != null) {
        value = datasetId.toString();
      }

      switch (parameterInfo.type) {
        case ParameterType.query:
          url.searchParams.set(parameterName, value);
          break;
        case ParameterType.header:
          myHeaders[parameterName] = value;
          break;
        case ParameterType.cookie:
          if (myHeaders.Cookie != null) {
            myHeaders.Cookie = myHeaders.Cookie + `; ${parameterName}=${value}`;
          } else {
            myHeaders.Cookie = `${parameterName}=${value}`;
          }
      }
    }

    const requestHref = url.href;
    let result: any = null;

    try {
      const response = await this.getRequestPromiseByHttpMethod(httpMethod as HttpMethod, requestHref, myHeaders);

      switch (this.getTagByPathName(pathName, httpMethod)) {
        case OpenApiTag.search:
          result = this.findListId(response, pathName, preview);
          this.addRequestInHistorical(requestHref);
          break;
        case OpenApiTag.dataset:
          result = this.processDataSet(response, pathName, preview);
          break;
      }
    } finally {
      this.spinners.set(pathName, false);
    }

    return result;
  }

  getRequestPromiseByHttpMethod(httpMethod: HttpMethod, requestUrl: string, headers: any): Promise<any> {
    let requestObs: Observable<any>;

    switch (httpMethod.toLowerCase()) {
      case HttpMethod.GET:
        requestObs = this.httpClient.get(requestUrl, { headers });
    }

    return requestObs.toPromise();
  }

  processDataSet(dataSet: any, pathName: string, preview: boolean) {
    if (preview) {
      this.previews.set(pathName, dataSet);
    } else {
      return dataSet;
    }
  }

  findListId(dataSetList: any, pathName: string, preview: boolean) {
    const dataSetIdPath = this.getDatasetIdPath();
    const dataSetIdList: number[] = [];
    this.getIdInResponse(dataSetList, dataSetIdPath, dataSetIdList);

    if (preview) {
      this.previews.set(pathName, dataSetIdList);
    } else {
      return dataSetIdList;
    }
  }

  getIdInResponse(data: any, dataSetIdPath: string[], dataSetIdList: number[]) {
    if (dataSetIdPath.length === 0) {
      return;
    }

    const currentPath = dataSetIdPath[0];

    if (!data.hasOwnProperty(currentPath) || data[currentPath] == null) {
      return;
    }

    if (typeof data[currentPath] !== 'object') {
      dataSetIdList.push(data[currentPath]);
      return;
    }

    if (!Array.isArray(data[currentPath])) {
      this.getIdInResponse(data[currentPath], dataSetIdPath.slice(1), dataSetIdList);
    } else {
      for (const index of data[currentPath]) {
        this.getIdInResponse(index, dataSetIdPath.slice(1), dataSetIdList);
      }
    }
  }

  getDatasetIdPath(): string[] {
    for (const path of this.openApi.paths) {
      for (const request of path.requests) {
        if (request.tags[0] !== OpenApiTag.search) {
          continue;
        }

        for (const response of request.responses) {
          if (!response.httpStatusCode.toString().startsWith('2')) {
            continue;
          }

          for (const content of response.contents) {
            if (content.contentType.toLowerCase() !== 'application/json') {
              continue;
            }

            if (content.example == null || content.example.length === 0) {
              continue;
            }

            return content.example.split('#');
          }
        }
      }
    }

    return [];
  }

  getTagByPathName(pathName: string, httpMethod: string): OpenApiTag {
    for (const path of this.openApi.paths) {
      if (path.pathName !== pathName) {
        continue;
      }

      for (const request of path.requests) {
        if (request.httpmethod === httpMethod) {
          return OpenApiTag[request.tags[0]];
        }
      }
    }

    return null;
  }

  openDatasetsSelectDialog(stepper: NbStepperComponent) {
    this.loading = true;
    this.stepper = stepper;

    let searchRequestPathName: string;
    let searchRequestHttpMethod: string;

    let datasetRequestPathName: string;
    let datasetRequestHttpMethod: string;

    for (const path of this.openApi.paths) {
      for (const request of path.requests) {
        if (request.tags == null || request.tags.length === 0) {
          continue;
        }

        switch (request.tags[0]) {
          case OpenApiTag.search:
            searchRequestPathName = path.pathName;
            searchRequestHttpMethod = request.httpmethod;
            break;
          case OpenApiTag.dataset:
            datasetRequestPathName = path.pathName;
            datasetRequestHttpMethod = request.httpmethod;
            break;
        }
      }
    }

    this.getListFromDialog(searchRequestPathName, searchRequestHttpMethod,
      datasetRequestPathName, datasetRequestHttpMethod);
    /*
      if (response.status.toString().startsWith('2'))
              this.dataSetService.saveDatasets(data);
    */
  }

  private getListFromDialog(searchRequestPathName: string, searchRequestHttpMethod: string,
                            datasetRequestPathName: string, datasetRequestHttpMethod: string) {
    this.launchRequest(searchRequestPathName, searchRequestHttpMethod, false).then((response) => {
      this.dialog.open(DatasetsDialogComponent, {
        width: '100vw',
        data: {
          datasetId: response
        },
      }).afterClosed().subscribe({
        next: (datasetId) => this.onCloseDatasetDialog(datasetRequestPathName, datasetRequestHttpMethod, datasetId)
      });
    }).finally(() => this.loading = false);
  }

  private onCloseDatasetDialog(datasetRequestPathName: string, datasetRequestHttpMethod: string, datasetId: number[]) {
    if (datasetId == null || datasetId.length === 0) {
      this.ready = false;
      return;
    }

    this.loading = true;

    const requestPromises: Promise<any>[] = [];

    for (const id of datasetId) {
      const requestPromise = this.launchRequest(datasetRequestPathName, datasetRequestHttpMethod, false, id)
        .then((dataset) => {
          this.dataSetService.saveDatasets(dataset, id);
        });
      requestPromises.push(requestPromise);
    }

    Promise.all(requestPromises).then(() => {
      this.ready = true;
      this.stepper.next();
    }).finally(() => this.loading = false);
  }

  private updateRequestHistorical() {
    this.requestHistorical = [];
    const catId = this.openApi.info['x-catalog-id'];

    if (catId == null) {
      return;
    }

    this.dataSetService.getRequestHistorical(catId).subscribe({
      next: (response) => {
        this.requestHistorical = [];
        response.map((el) => { this.requestHistorical.push(el.request); });
      }
    });
  }

  private addRequestInHistorical(request: string) {
    const catId = this.openApi.info['x-catalog-id'];

    if (catId == null) {
      return;
    }

    this.dataSetService.createRequestHistorical(catId, request).subscribe({
      next: () => this.updateRequestHistorical()
    });
  }

  /*listdatasets() {
    var myHeaders = new Headers();
    myHeaders.append("Content-Type", "Application/json");
    var myInit = { method: 'GET', headers: myHeaders };


    //appeler smart havester pour récuperer l'api à lancer


    var myRequest = new Request('https://' + this.urlRepo + '/api/search?q=*&per_page=5&type=dataset&start=50&show_facets=true&show_my_data=true', myInit);

    fetch(myRequest, myInit)
      .then(response => {
        response.json()
          .then(data => {
            this.itemsdatasets = data['data']['items'];
            // console.log(this.itemsdatasets);
            for (var i = 0; i < this.itemsdatasets.length; i++) {
              if (!this.dataSetService.findDataSetByTitle(data['data']['items'][i]['name'])) {
                if (data['data']['items'][i]['global_id']) {
                  this.populatecatalogue(data['data']['items'][i]['global_id']);
                }
              }
            }
          });
      });
    return null;
  }


  populatecatalogue(id: string) {
    var myHeaders = new Headers();
    myHeaders.append("Content-Type", "Application/json");
    var myInit = { method: 'GET', headers: myHeaders };

    //appeler smart havester pour récuperer l'api à lancer

    var myRequest = new Request('https://' + this.urlRepo + '/api/datasets/export?exporter=dataverse_json&persistentId=' + id, myInit);

    fetch(myRequest, myInit)
      .then(response => {
        response.json()
          .then(data => {
            this.itemsdataset = data;

            console.log(this.itemsdataset);

            this.createdataset(data);
            //this.createdistribution(this.itemsdataset);                       
          });
      });




  }




  createdataset(item: any) {
    //console.log(item);
    let data: string;
    let description = "";
    let name = "";
    let url: string;

    for (var i = 0; i < item['datasetVersion']['metadataBlocks']['citation']['fields'].length; i++) {
      if (item['datasetVersion']['metadataBlocks']['citation']['fields'][i]['typeName'] == "dsDescription") {
        description = JSON.stringify(item['datasetVersion']['metadataBlocks']['citation']['fields'][i]['value']);
      }
      if (item['datasetVersion']['metadataBlocks']['citation']['fields'][i]['typeName'] == "title") {
        name = JSON.stringify(item['datasetVersion']['metadataBlocks']['citation']['fields'][i]['value']);
      }
    }

    url = JSON.stringify(item['persistentUrl']);

    data = '\@prefix dcat: <http://www.w3.org/ns/dcat#>.\n\
            @prefix dct: <http://purl.org/dc/terms/>.\n\
            @prefix language: <http://id.loc.gov/vocabulary/iso639-1/>.\n\
            @prefix s: <'+ this.FDP_URL + '/>.\n\
            @prefix c: <'+ this.FDP_URL+ '/catalog/>.\n\
    s:new\n\
      a dcat:Dataset, dcat:Resource;\n\
      dct:description ' + description + ';\n\
      dct:hasVersion "1.0";\n\
      dct:isPartOf <https://f2ds.eosc-pillar.eu/catalog/afe472f9-2e70-409c-ad26-f174799e7834>;\n\
      dct:language language:en;\n\
      dct:license <http://rdflicense.appspot.com/rdflicense/cc-by-nc-nd3.0>;\n\
      dct:title '+ name + ';\n\
      dcat:keyword '+ url + '.\n'


    console.log(data);
    // return this.dataSetService.createDataSet(data);

  }



  createdistribution(item: any) {

    // si file alors download url https://dataverse.ird.fr/api/access/datafile/['files']['dataFile'][i]['id']]

    console.log(item);
    let data: string;
    let description: string;
    description = JSON.stringify(item['description']);
    let name: string;
    name = JSON.stringify(item['name']);
    let url: string;
    url = JSON.stringify(item['url']);

    data = '@prefix dcat: <http://www.w3.org/ns/dcat#>.\n\
  @prefix dct: <http://purl.org/dc/terms/>.\n\
  @prefix language: <http://id.loc.gov/vocabulary/iso639-1/>.\n\
  @prefix ffd: <https://f2ds.eosc-pillar.eu/>.\n\
  @prefix d: <https://f2ds.eosc-pillar.eu/dataset/>.\n\
  @prefix GVW: <https://dataverse.ird.fr/dataset.xhtml?persistentId=doi:10.23708/GVWCA0#>.\n\
  \n\
  ffd:new\n\
      a dcat:Distribution, dcat:Resource;\n\
      dct:description "Botinelli_VNM_2016.png ";\n\
      dct:hasVersion "v1.0";\n\
      dct:isPartOf d:ca1b25a9-901c-4822-8554-b1bb416ee99d;\n\
      dct:language language:fr;\n\
      dct:license <http://rdflicense.appspot.com/rdflicense/cc-by-nc-nd3.0>;\n\
      dct:title "Botinelli_VNM_2016.png ";\n\
      dcat:accessURL GVW:;\n\
      dcat:downloadURL\n\
      <https://dataverse.ird.fr/api/access/datafile/262?imageThumb=400>;\n\
      dcat:format "png";\n\
      dcat:mediaType "png".'


    return this.dataSetService.createDistribution(data);

  }*/





}
